/**
 * 访问Android或者iOS本地设备API
 * @class $.maf.native
 */
 
(function( $ ){
	
	/**
	 * 文件下载API，包括如下功能：
	 * 
	 * - 下载远程服务器的文件
	 * - 将文件上传到远程服务器上
	 * - 取消正在进行的文件上传/下载
	 * - 安装apk文件
	 * - 软件版本管理控制
	 * 
	 * {@img native/update.png 文件下载API}
	 * 
	 * 样例如下所示：
	 * 
	 * 		//下载文件
	 * 		function downLoadFile(){
	 * 			var downloader = $.maf.native.downloader,
	 * 				downloaderHandler,
	 * 				successFn = function( res ){
	 * 					console.log(JSON.stringify(res));
	 * 				},
	 * 				errorFn = function( err ){
	 * 					console.log(JSON.stringify(err));
	 * 				};
	 * 
	 * 			downloaderHandler = downloader.downloadFile( "http://localhost/osc.apk", {}, successFn, errorFn );
	 * 			//取消下载
	 * 			downloader.cancelLoad(downloaderHandler);
	 * 		}
	 * 
	 * 		//上传文件
	 * 		function upLoadFile(){
	 * 			var downloader = $.maf.native.downloader,
	 * 				uploaderHandler,
	 * 				successFn = function( res ){
	 * 					console.log(JSON.stringify(res));
	 * 				},
	 * 				errorFn = function( err ){
	 * 					console.log(JSON.stringify(err));
	 * 				};
	 *
	 * 			uploaderHandler = downloader.uploadFile("/mnt/sdcard/MAF_Android.apk", 
	 * 						"http://10.4.45.90/upload/upload_file.php", successFn, errorFn, {} );
	 * 			//取消上传
	 * 			downloader.cancelLoad(uploaderHandler);
	 * 		}
	 * 
	 * 
	 * @class $.maf.native.downloader
	 */
	cordova.define("maf/downloader", function(require, exports, module){
		
		/**
		 * 下载文件到指定目录
		 * @param {String} url 文件下载地址
		 * @param {Object} params 下载参数。有如下可选值
		 * 
		 * - fileName 文件名称。如不设置则通过url地址判断
		 * - dirName 父文件夹名称。如不设置则为SD卡的download文件夹（只针对Android）
		 * 
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - status 文件下载状态。0表示下载中，1表示下载完成
		 * - total 文件大小。文件下载完成后，该参数可能为空
		 * - progress 文件下载进度。文件下载完成后，该参数可能为空
		 * - fileName 文件名称。文件下载过程中，该参数可能为空
		 * - filePath 文件全路径。文件下载过程中，该参数可能为空
		 * 
		 * @param {Function} errorFn 失败回调函数，函数会传入如下参数
		 * 
		 * - httpStatusCode 服务端响应码
		 * - errorCode 错误码 （1: FILE_NOT_FOUND_ERR; 2: INVALID_URL_ERR; 3: CONNECTION_ERR; 4: ABORTED_ERR）
		 * 
		 * @return {Object} 返回文件下载过程的句柄

		 * @method downloadFile
		 */
		exports.downloadFile = function( url, params, successFn, errorFn ){
			!params && ( params = {} );
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			
			var fileTransfer = new FileTransfer(),
				uri = encodeURI(url),
				fileName = params.fileName ? params.fileName : url.substring(url.lastIndexOf("/") + 1),
				//dirName需要完善，以兼容IOS
				dirName = params.dirName ? params.dirName : $.maf.getDownloadPath(),
				target = dirName + "/" + fileName;

			var progressFn = function( evt ){
				var obj = { status: 0, total: evt.total, progress: Math.floor((evt.loaded/evt.total).toFixed(2)*100) };
				successFn( obj );
			};
			
			var completeFn = function( data ){
				var obj = { status: 1, fileName: data.name, filePath: data.fullPath };
				successFn( obj );
			};
			
			var downloadErrorFn = function( err ){
				var obj = { httpStatusCode: err.http_status, errorCode: err.code };
				errorFn( obj );
			};
			
			fileTransfer.onprogress = progressFn;
			fileTransfer.download( uri, target, completeFn, downloadErrorFn );
			return fileTransfer;
		}
		
		/**
		 * 取消文件上传/下载
		 * @param {Object} loadHandler 文件上传/下载过程的句柄
		 * @method cancelLoad
		 */
		exports.cancelLoad = function( loadHandler ){
			if (loadHandler) {
				loadHandler.abort();
				loadHandler = null;
			}
		}
		
		/**
		 * 上传文件到指定的服务器上
		 * @param {String} filePath 文件在设备上的完整路径
		 * @param {String} serverUrl 文件上传的服务器地址
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - status 文件上传状态。0表示上传中，1表示上传完成
		 * - total 文件大小。文件上传完成后，该参数可能为空
		 * - progress 文件上传进度。文件上传完成后，该参数可能为空
		 * - bytesSent 上传字节数。文件上传过程中，该参数可能为空
		 * 
		 * @param {Function} errorFn 失败回调函数，函数会传入如下参数
		 * 
		 * - httpStatusCode 服务端响应码
		 * - errorCode 错误码 （1: FILE_NOT_FOUND_ERR; 2: INVALID_URL_ERR; 3: CONNECTION_ERR; 4: ABORTED_ERR）
		 * 
		 * @param {Object} params 上传参数。有如下可选值
		 * 
		 * - fileKey 表单元素名称。如不设置则默认为”file“
		 * - fileName 文件名称。如不设置则通过filePath判断
		 * - mimeType 文件类型。如不设置则默认为”application/octet-stream“。具体设置值可参考**[MIME类型](http://www.w3school.com.cn/media/media_mimeref.asp)**
		 * - headers header名和header值的映射
		 * 
		 * @return {Object} 返回文件上传过程的句柄
		 * 
		 * @method uploadFile
		 */
		exports.uploadFile = function( filePath, serverUrl, successFn, errorFn, params ){
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			!params && ( params = {} );

			var fileTransfer = new FileTransfer();
			var options = {};
			
			options.fileKey = params.fileKey ? params.fileKey : "file";
			options.fileName = params.fileName ? params.fileName : filePath.substr(filePath.lastIndexOf('/') + 1);
			options.mimeType = params.mimeType ? params.mimeType : "application/octet-stream";
			options.headers = params.headers;

			var progressFn = function( evt ){
				var obj = { status: 0, total: evt.total, progress: Math.floor((evt.loaded/evt.total).toFixed(2)*100) };
				successFn( obj );
			};
			
			var completeFn = function( data ){
				var obj = { status: 1, bytesSent: data.bytesSent };
				successFn( obj );
			};
			
			var uploadErrorFn = function( err ){
				var obj = { httpStatusCode: err.http_status, errorCode: err.code };
				errorFn( obj );
			};
			
			fileTransfer.onprogress = progressFn;
			fileTransfer.upload( filePath, encodeURI(serverUrl), completeFn, uploadErrorFn, options, true );
			return fileTransfer;			
		}

		/**
		 * 安装apk文件
		 * @param {String} dirName 父文件夹名称
		 * @param {String} fileName 文件名称
		 * @method installApk
		 */
		exports.installApk = function( dirName, fileName ){
			cordova.exec(null,null,"MAF_Downloader","installApk",[ dirName, fileName ] );
		}

		/**
		 * 版本管理
		 * 
		 * 检测是否有最新版本的升级文件；如果有，提示用户下载更新。具体使用方法请参考**[versionControl](#!/api/$.maf-property-versionControl)**
		 * 
		 * @method update
		 */
		exports.update = function(){
			 var constant = cordova.require("maf/Constants"),
                updateUrl = constant.updateUrl,
                currentVersion = constant.versionInfo.versionCode,
				argLen = arguments.length,
				apkUrl,
				startUpdate = function( res ){
					res == "1" && cordova.exec( null, null,"MAF_Downloader","update",[ apkUrl ] );
				};

			$.ajax({url: updateUrl, dataType: "json", cache: false}).success(function(res){
				var remoteVersion = res["version"];
				apkUrl  = res["url"];
               
				parseFloat(remoteVersion) > parseFloat(currentVersion) ?
					cordova.exec( startUpdate, null, "Notification", "confirm", ["发现新版本，是否更新？","版本更新", ["确定","下次再说"]]):
					argLen === 0 && cordova.exec( null, null,  'MAF_ToastUtil', 'showToast', [ "未检测到新版本", 0, 2 ] );
			});
		}
		
	});
	
	
	/**
	 * 原生日期API，包括如下功能：
	 * 
	 * - 日期选择框
	 * - 时间选择框
	 * 
	 * {@img native/datepicker.png 原生日期API}
	 * 
	 * 样例代码如下：
	 * 		$.maf['native'].datepicker.showDate( 
	 * 				function(d){
	 * 					alert(JSON.stringify(d));
	 * 				}, function(){
	 * 					alert("Failed to show date!");
	 * 				});
	 * 		$.maf['native'].datepicker.showTime(
	 * 				function(t){
	 * 					alert(JSON.stringify(t));
	 * 				}, function(){
	 * 					alert("Failed to show time!");
	 * 				});
	 * 
	 * @class $.maf.native.datepicker
	 */
	cordova.define("maf/datepicker", function(require, exports, module){
	
		/**
		 * 显示一个原生的日期框
		 * @param {Function} successFn 成功回调，  函数会传入如下参数
		 * 
		 * - year 年
		 * - month 月 ( 0-11 )
		 * - day 日 ( 1-31 )
		 * 
		 * @param {Function} errorFn 失败回调
		 * @method showDate
		 */
		exports.showDate = function( successFn, errorFn ){
			cordova.exec( successFn || null, errorFn || null, 'MAF_DateUtil', 'date', [] );
		}
		
		/**
		 * 显示一个原生的时间选择框
		 * @param {Function} successFn 成功回调，  函数会传入如下参数
		 * 
		 * - hour 时 ( 0-23 )
		 * - minute 分 ( 0-59 )
		 * 
		 * @param {Function} errorFn 失败回调
		 * @method showTime
		 */
		exports.showTime = function( successFn, errorFn ){
			cordova.exec( successFn || null, errorFn || null, 'MAF_DateUtil', 'time', [] );
		}
		
	});
	
	
	/**
	 * 应用程序退出控制API
	 * 
	 * **注意**: 只适合在Android上使用
	 * 
	 * {@img native/appexit.png 应用程序退出控制API}
	 * 
	 * 样例代码如下：
	 * 		$.maf['native'].exitApp.exitApp("注销登录","你确定要注销登录吗？");
	 * 
	 * @class $.maf.native.exitApp
	 */
	cordova.define("maf/exitApp", function(require, exports, module){
		
		/**
		 * 退出应用程序时显示提示框
		 * @param {String} [message=""] 提示框的内容
		 * @param {String} [title=""] 提示框的标题名称
		 * @param {Array} [buttonLabels=["确定","取消"]] 提示框按钮名称，多个按钮用逗号分隔
		 * 
		 * **注意**：有多个按钮时，点击第一个按钮程序会退出，其他按钮会关闭对话框
		 */
		exports.exitApp = function( message,  title, buttonLabels ){
			cordova.exec(
					function(result){result == 1 && cordova.exec(null, null, "App", "exitApp", [])}, 
					null, 
					"Notification",
					"confirm", 
					[message ||"", title|| "", buttonLabels || ["确定","取消"]]
			);
			
		}
		
	});
	
	
	/**
	 * 信息提示API，它包含如下功能
	 * 
	 * - 显示提示信息(类似于Android的Toast提示）
	 * - 关闭提示信息
	 * - 震动
	 * - 显示页面加载提示(showLoading)
	 * - 关闭页面加载提示
	 * - 显示进度条信息
	 * - 设置进度条的值
	 * - 关闭进度条
	 * - 在状态栏上显示提示信息(Android和iOS上的下拉信息提示)
	 * 
	 * {@img native/notification.png 应用程序退出控制API}
	 * 
	 * 样例代码如下：
	 * 		//显示加载提示
	 * 		$.maf['native'].notification.showLoading("正在加载信息，请稍后……");
	 * 
	 * 		//关闭加载提示
	 * 		$.maf['native'].notification.hideLoading();
	 * 
	 * 		//显示、更新、关闭进度条
	 * 		$.maf['native'].notification.showProgress("加载进度","当前进度为");
	 * 		var i = 0;
	 * 		var interval = setInterval(function(){
	 * 				i++;
	 * 				if(i <= 100){
	 * 					$.maf['native'].notification.setProgressValue(i);
	 * 				}else{
	 * 					$.maf['native'].notification.hideProgress();
	 * 					clearInterval(interval);
	 * 				}
	 * 			}, 150);
	 * 
	 * 		//在状态栏上显示提示信息
	 * 		$.maf['native'].notification.notify("notify1", "通知", "这是一条通知消息");
	 * 		//清除提示信息
	 * 		$.maf['native'].notification.clearNotify("notify1");
	 *  
	 * @class $.maf.native.notification
	 */
	cordova.define("maf/notification", function(require, exports, module){
		
		// 调用showLoading或者hideLoading的次数
		var loadingCount = 0;
		
		/**
		 * 在屏幕下方显示提示信息
		 * @param {String} text 文字信息
		 * @param {Number} duration 显示时间，可选值为0或者1
		 * 
		 * - 0 显示较短时间
		 * - 1 显示较长时间
		 * 
		 * @param {Number} [position=2] 设置文字显示位置，可选值为0、1、2
		 * 
		 * - 0 显示在页面顶端
		 * - 1 显示在页面中间
		 * - 2 显示在页面底部
		 * 
		 * @method showToast
		 */
		exports.showToast = function( text, duration, position ){
			typeof( duration ) === "undefined" && ( duration = 0 );
			typeof( position ) === "undefined" && ( position = 2 );
			cordova.exec( null, null,  'MAF_ToastUtil', 'showToast', [ text, duration, position ] );
		}
		
		/**
		 * 关闭正在显示的提示信息
		 * @method hideToast
		 */
		exports.hideToast = function(){
			cordova.exec( null, null,  'MAF_ToastUtil', 'hideToast', [] );
		}
		
		/**
		 * 震动提示
		 * @param {Number} [time=500] 震动多少毫秒
		 * @method vibrate
		 */
		exports.vibrate = function( time ){
			typeof( time ) === "undefined" && ( time = 550 );
			cordova.exec( null, null,  'Notification', 'vibrate', [ time ] );
			
		}
		
		
		/**
		 * 显示加载提示框，显示loading效果
		 * @param {String} title
		 * @param {String} message
		 * @method showLoading
		 */
		exports.showLoading = function( title, message ){
			if( loadingCount !== 0 ) {
				loadingCount++ ;
				return;
			}
			arguments.length == 1 &&  ( message = title ) && ( title = "");
			cordova.exec(null,null,"MAF_Notification","showLoading",[ title, message ] );
			loadingCount++;
		}
		
		/**
		 * 关闭加载提示框
		 * @method hideLoading
		 */
		exports.hideLoading = function(){
			loadingCount--;
			// 防止用户没调用showLoding而直接调用了hideLoading
			if( loadingCount <= 0 ){
				cordova.exec(null,null,"MAF_Notification","hideLoading",[ ] );
				loadingCount = 0;
			}
		}
		
		/**
		 * 显示进度条信息
		 * @param {String} title
		 * @param {String} message
		 * @method showProgress
		 */
		exports.showProgress = function( title, message){
			arguments.length == 1 &&  ( message = title ) && ( title = "" )
			cordova.exec(null,null,"MAF_Notification","showProgress",[ title, message ] );
		}
		
		/**
		 * 设置进度条的值
		 * @param {Number} value
		 * @method setProgressValue
		 */
		exports.setProgressValue = function( value ){
			typeof(value) !="number" && ( value = 0);
			!value && ( value = 0);
			cordova.exec(null,null,"MAF_Notification","setProgressValue",[ value ] );
		}
		
		/**
		 * 关闭进度条
		 * @method hideProgress
		 */
		exports.hideProgress = function(){
			cordova.exec(null,null,"MAF_Notification","hideProgress",[  ] );
		}
		
		/**
		 * 显示通知消息，显示在状态栏上，下拉可以看见
		 * @param {String} tag 唯一标示符
		 * @param {String} title 消息标题
		 * @param {String} message 消息内容
		 * @method notify
		 */
		exports.notify = function( tag, title, message ){
			cordova.exec( null, null, "MAF_StatusBarNotification", "notify", [ tag, title, message, 16]);
		}
		
		/**
		 * 清除状态栏上指定的通知
		 * @param {String} tag
		 * @method clearNotify
		 */
		exports.clearNotify = function( tag ){
			cordova.exec( null, null, "MAF_StatusBarNotification", "clearNotify", [ tag ]);
		}
		
		/**
		 * 清除状态栏上所有的通知信息
		 * @method clearAllNotify
		 */
		exports.clearAllNotify = function(){
			cordova.exec( null, null, "MAF_StatusBarNotification", "clearAllNotify", []);
		}
		
		
		/**
		 * 显示原生对话框
		 * @param {String} message 要显示的消息
		 * @param {Function} alertCallback 点击按钮后的回调函数
		 * @param {String} title 信息标题
		 * @param {String} buttonName 按钮名称
		 * @method alert
		 */
		exports.alert = function( message, alertCallback, title, buttonName ){
			navigator.notification.alert(message, alertCallback, title, buttonName)
		}
		
		/**
		 * 显示原生确认框
		 * @param {String} message 要显示的消息
		 * @param {Function} confirmCallback 点击按钮后的回调函数，回调传入的参数值分别为1、2、3...，即点击第N个按钮，传入的值就是N。如果直接点击返回按钮，返回为0
		 * @param {String} title 信息标题
		 * @param {String} buttonLabels  按钮名称，多个按钮以逗号隔开
		 * @method confirm
		 */
		exports.confirm = function( message, confirmCallback, title, buttonLabels ){
			
			navigator.notification.confirm(message, confirmCallback, title, buttonLabels);
		}
		
		
	});
	
	
	/**
	 * 设备处理API，有如下功能
	 * 
	 * - 禁用回退按钮（仅对Android设备有效）
	 * - 解禁回退按钮（仅对Android设备有效）
	 * - 控制设备旋转方向
	 * - 获取当前应用程序版本信息
	 * - 获取设备信息
	 * 
	 * 样例代码如下：
	 * 		//获取当前应用程序版本信息
	 * 		var v = $.maf['native'].device.getVersion();
	 * 		alert("SW version is " + JSON.stringify(v));
	 * 
	 * 		//获取设备信息
	 * 		var Info = $.maf['native'].device.getDeviceInfo();
	 * 		alert("Device Info is " + JSON.stringify(Info));
	 * 
	 * 		//禁用回退按钮
	 * 		$.maf['native'].device.disableBackbutton();
	 * 		//解禁回退按钮
	 * 		$.maf['native'].device.enableBackbutton();
	 * 
	 * 		//控制设备旋转方向--横向，Home键向右
	 * 		$.maf['native'].device.setOrientation(0);
	 *  
	 * @class $.maf.native.device
	 * 
	 */
	cordova.define("maf/device", function(require, exports, module){
		
		/**
		 * 禁用返回按钮（仅对Android设备有效）
		 * @method disableBackbutton
		 */
		exports.disableBackbutton = function(){
			cordova.exec(null, null, "App", "overrideBackbutton", [true]);
		}
		
		/**
		 * 解禁返回按钮（仅对Android设备有效）
		 * @method enableBackbutton
		 */
		exports.enableBackbutton = function(){
			cordova.exec(null, null, "App", "overrideBackbutton", [false]);
		}
		
		/**
		 * 控制设备旋转方向
		 * 
		 * @param {Number} [orientation="1"] 有如下可选值
		 * 
		 * - 0 横向，Home键向右 
		 * - 1 竖直，Home键向下
		 * - 8 横向倒转，Home键向左
		 * - 9 垂直倒转，Home键向上
		 * 
		 * @method setOrientation
		 */
		exports.setOrientation = function( orientation ){
			typeof( orientation ) == "undefined" && ( orientation == 1);
			cordova.exec( null, null, "MAF_Device", "setOrientation", [ orientation ])
		}
		
		/**
		 * 获取当前软件版本信息
		 * @method getVersion
		 * @return {Object}
		 * 数据结构如下：
		 * 
		 * - versionCode 软件版本名号，一般为数字，常用于软件升级，比如win7某个版本的versionCode就是6.1.7601
		 * - verionName 软件名称版本号，一般用于用户展示，比如win7、win8
		 */
		exports.getVersion = function( ) {
			var constant = cordova.require("maf/Constants");
			return constant.versionInfo;
		}
		
		/**
		 * 获取系统硬件设备信息
		 * @method getDeviceInfo
		 * @return {Object} 返回如下信息
		 * 
		 * - name 设备的名称，比如htc、GT-9100G等
		 * - platform 设备的平台信息，如Android、iOS等
		 * - uuid 设备的硬件唯一标示符
		 * - version 操作系统版本号
		 * - cordova Cordova/PhoneGap版本号
		 */
		exports.getDeviceInfo = function(){
			var obj =  cordova.require('cordova/plugin/device');
			delete obj.available;
			return obj;
		}

	});


	/**摇一摇API
	 * 
	 * 监测设备的摇动，摇动开始、结束时分别进行执行回调函数
	 * 
	 * 样例代码如下：
	 * 		var watchHandler = $.maf['native'].shaker.watchShake( 
	 * 					function(){
	 * 						$("#hint1").text("开始摇动！");
	 * 					}, function(){
	 * 						$("#hint2").text("摇动结束！");
	 * 					});
	 * 
	 * 		//取消摇动监测
	 * 		$.maf['native'].shaker.cancelWatch(watchHandler);
	 * 
	 * @class $.maf.native.shaker
	 */

	cordova.define("maf/shaker", function(require, exports, module){
		/**
		 * 监测设备的摇动
		 * 
		 * @param {Function} onShakeBegin 检测到摇动开始的回调函数
		 * 
		 * @param {Function} onShakeEnd 检测到摇动结束的回调函数
		 * 
		 * @return {Object} 返回监测过程的句柄
		 * 
		 * @method watchShake
		 * 
		 */
		exports.watchShake = function( onShakeBegin, onShakeEnd ){
			
			onShakeBegin = $.isFunction( onShakeBegin ) ? onShakeBegin : $.noop;
			onShakeEnd = $.isFunction( onShakeEnd ) ? onShakeEnd : $.noop;

			var watchId = null,
				isShaking = false,
				prevAccel = {},
				accelChange = {},
				delta = 0,
				//摇动开始阈值，x、y、z三方向加速度变化量的向量和。
				beginThreshold = 15,
				//摇动结束阈值，x、y、z三方向加速度变化量的向量和。
				endThreshold = 1,
				//监测频率（milliseconds）
				freq = 200;
				
			var assessAccelChange = function(currentAccel){
				
				if ( typeof( prevAccel.x ) !== "undefined" ) {
					accelChange.x = Math.abs(currentAccel.x - prevAccel.x);
					accelChange.y = Math.abs(currentAccel.y - prevAccel.y);
					accelChange.z = Math.abs(currentAccel.z - prevAccel.z);
				}
				//比较x、y、z三方向加速度变化量的向量和
				delta = Math.sqrt(accelChange.x * accelChange.x + accelChange.y * accelChange.y + accelChange.z * accelChange.z);
				prevAccel = { x: currentAccel.x, y: currentAccel.y, z: currentAccel.z };
				if( delta > beginThreshold ) {
					!isShaking && ( isShaking = true ) && onShakeBegin();
				}else if( delta < endThreshold ) {
					isShaking && !( isShaking = false ) && onShakeEnd();
				}
			};

			watchId = navigator.accelerometer.watchAcceleration(assessAccelChange, null, {frequency : freq});
				
			return watchId;
		}
		
		/**
		 * 取消摇动监测
		 * 
		 * @param {Object} watchId 监测过程的句柄
		 * 
		 * @method cancelWatch
		 */
		exports.cancelWatch = function( watchId ){
			if (watchId) {
				navigator.accelerometer.clearWatch(watchId);
				watchId = null;
			}
		}		
		
	});
	
	
	
	/**
	 * 发送短信息API
	 * @class $.maf.native.sms
	 */
	cordova.define("maf/sms", function(require, exports, module){
		
		/**
		 * 发送短信
		 * @param {String} number 短信号码
		 * @param {String} message 消息内容
		 * @param {Function} successFn 短信发送成功回调
		 * @param {Function} errorFn 短信发送失败回调
		 * @method sendSMS
		 */
	 	exports.sendSMS = function( number, message,successFn, errorFn){
	 	   cordova.exec( successFn || null, errorFn || null, "MAF_SMS", "sendSMS", [ number, message ]);
	 	}
		
	});
	

	/**
	 * 设备定位服务API，有如下功能
	 * 
	 * - 获取设备定位
	 * - 对设备定位进行监控
	 * - 取消定位监控
	 * - 检测设备的GPS开启状态（目前仅限Android）
	 * - 打开设备的GPS设置页面（目前仅限Android）
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			//检测设备的GPS开启状态
	 * 			var gps = $.maf['native'].gps.isGPSOpen();
	 * 
	 * 			//打开设备的GPS设置页面
	 * 			$.maf['native'].gps.configGPS();
	 * 
	 * 			//获取设备定位
	 * 			$.maf['native'].gps.getCurrentPosition(
	 * 					function(x){
	 * 						$("#gpsInfo").text( "设备定位" + JSON.stringify(x) );
	 * 					}, function(err){
	 * 						$("#gpsInfo").text( "设备定位错误" + JSON.stringify( err ) );
	 * 					}, { timeout:60000, maximumAge:0 } );
	 * 
	 * 			//对设备定位进行监控
	 * 			var watchID = $.maf['native'].gps.watchPosition(
	 * 					function(x){
	 * 						$("#gpsWatchInfo").text( "监控定位" + JSON.stringify(x) + ". watchID is " + watchID );
	 * 					}, function(err){
	 * 						$("#gpsWatchInfo").text( "监控定位错误" + JSON.stringify( err ) );
	 * 					}, { timeout:20000, maximumAge:60000 } );
	 * 
	 * 			//取消定位监控
	 * 			$.maf['native'].gps.clearWatch(watchID);
	 * 		</script>
	 * 
	 * @class $.maf.native.gps
	 */
	cordova.define("maf/gps", function(require, exports, module){

		var isGPSOpen = function() {
			return cordova.exec( null, null, "MAF_Device", "isGPSOpen", [] );
		}
		
		/**
		 * 获取设备定位
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - latitude 纬度
		 * - longitude 经度
		 * 
		 * @param {Function} errorFn 失败回调函数，函数会传入如下参数
		 * 
		 * - message 失败消息
		 * - code 错误码。1、PERMISSION_DENIED；2、POSITION_UNAVAILABLE；3、TIMEOUT
		 * 
		 * @param {Object} params 设定参数。有如下可选值
		 * 
		 * - maximumAge 缓存定位信息有效期（milliseconds）。若未设置则默认为0
		 * - timeout 超时时间（milliseconds）。该时间段内若未返回成功回调，则调用失败回调。若未设置则默认为2min
		 * 
		 * @method getCurrentPosition
		 */
		exports.getCurrentPosition = function( successFn, errorFn, params ){
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			!params && ( params = {} );
			
			if( isGPSOpen() == "false" ){
				cordova.exec( null, null, 'MAF_ToastUtil', 'showToast', [ "请开启GPS！", 0, 2 ] );
				return;
			}
			//enableHighAccuracy: true，GPS定位；enableHighAccuracy: false，AGPS定位
			var options = { maximumAge: params.maximumAge || 0, timeout: params.timeout || 120000, enableHighAccuracy: true },
				fn = function( obj ){
					var coords = obj.coords;
					successFn( {latitude: coords.latitude, longitude: coords.longitude} );
				};

			navigator.geolocation.getCurrentPosition( fn, errorFn, options );
		}

		/**
		 * 监控设备定位
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - latitude 纬度
		 * - longitude 经度
		 * 
		 * @param {Function} errorFn 失败回调函数，函数会传入如下参数
		 * 
		 * - message 失败消息
		 * - code 错误码。1、PERMISSION_DENIED；2、POSITION_UNAVAILABLE；3、TIMEOUT
		 * 
		 * @param {Object} params 设定参数。有如下可选值
		 * 
		 * - maximumAge 缓存定位信息有效期（milliseconds）。若未设置则默认为0
		 * - timeout 超时时间（milliseconds）。该时间段内若未返回成功回调，则调用失败回调。若未设置则默认为2min

		 * @return {String} 返回该监控过程的ID
		 * 
		 * @method watchPosition
		 */
		exports.watchPosition = function( successFn, errorFn, params ){
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			!params && ( params = {} );
			
			if( isGPSOpen() == "false" ){
				cordova.exec( null, null, 'MAF_ToastUtil', 'showToast', [ "请开启GPS！", 0, 2 ] );
				return;
			}

			//监控的频率在phoneGap中写死：GPS定位，一分钟更新一次。
			var options = { maximumAge: params.maximumAge || 0, timeout: params.timeout || 120000, enableHighAccuracy: true },
				fn = function( obj ){
					var coords = obj.coords;
					successFn( {latitude: coords.latitude, longitude: coords.longitude} );
				};

			return navigator.geolocation.watchPosition( fn, errorFn, options );
		}
		
		/**
		 * 取消定位监控
		 * 
		 * @param {String} watchID 监控过程的ID
		 * 
		 * @method clearWatch
		 */
		exports.clearWatch = function( watchID ){
			if (watchID != null) {
				navigator.geolocation.clearWatch( watchID );
				watchID = null;
			}
		}
		
		/**
		 * 检测设备的GPS开启状态（目前仅限Android）
		 * 
		 * @return {String} 当前GPS开启状态，有如下几种情况：
		 * 
		 * - false ：GPS未开启
		 * - true ：GPS已开启
		 * 
		 * @method isGPSOpen
		 */
		exports.isGPSOpen = isGPSOpen;

		/**
		 * 打开设备的GPS设置页面（目前仅限Android）
		 * @method configGPS
		 */
		exports.configGPS = function() {
			cordova.exec( null, null, "MAF_Device", "configGPS", [] );	 		
		}
	});

	/**
	 * 网络服务API，包括如下功能
	 * 
	 * - 获取网络连接状态
	 * - 打开设备的网络设置页面（目前仅限Android）
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			//获取网络连接状态
	 * 			var connect = $.maf['native'].connection.getConnection();
	 * 
	 * 			//打开设备的网络设置页面
	 * 			$.maf['native'].connection.configNetwork();
	 * 		</script>
	 * 
	 * @class $.maf.native.connection
	 */
	cordova.define("maf/connection", function(require, exports, module){
		
		/**
		 * 检测设备网络连接状态
		 * 
		 * @return {String} 当前网络状态，有如下几种情况：
		 * 
		 * - 2G
		 * - 3G
		 * - 4G
		 * - WiFi
		 * - Unknown：未知网络
		 * - NONE：未连接网络
		 *  
		 * @method getConnection
		 */
		exports.getConnection = function(){
			var networkState = navigator.connection.type;
			var states = {};

			states[Connection.UNKNOWN]	= 'Unknown';
			states[Connection.WIFI]		= 'WiFi';
			states[Connection.CELL_2G]	= '2G';
			states[Connection.CELL_3G]	= '3G';
			states[Connection.CELL_4G]	= '4G';
			states[Connection.NONE]		= 'NONE';
			
			return states[networkState];
		}
		
		/**
		 * 打开设备的网络设置页面（目前仅限Android）
		 * 
		 * @method configNetwork
		 */
		exports.configNetwork = function(){
			cordova.exec( null, null, "MAF_Device", "configNetwork", [] );
		}
	});

	
	/**
	 * 相机服务API，包括如下功能
	 * 
	 * - 打开相机拍照获取图片
	 * - 打开设备图片库获取图片
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			//打开相机拍照获取图片
	 * 			$.maf['native'].camera.takePicture( 
	 * 				function(x){
	 * 					var myImage = document.getElementById('myImage');
	 * 					myImage.src = x;
	 * 				}, function(err){
	 * 					alert( 'Failed because: ' + JSON.stringify( err ) );
	 * 				});
	 * 
	 * 	 		//打开设备图片库获取图片
	 * 			$.maf['native'].camera.getPicture( 
	 * 				function(x){
	 * 					var myImage = document.getElementById('myImage');
	 * 					myImage.src = x;
	 * 				}, function(err){
	 * 					alert( 'Failed because: ' + JSON.stringify( err ) );
	 * 				});
	 * 		</script>
	 * 
	 * @class $.maf.native.camera
	 */
	cordova.define("maf/camera", function(require, exports, module){
		

		/**
		 * 打开相机拍照获取图片
		 * 
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - imageData : 图像文件的URI。（字符串类型）
		 * 
		 * @param {Function} errorFn 错误回调函数，函数会传入如下参数
		 * 
		 * - message : 设备本地代码提供的错误信息。（字符串类型）
		 * 
		 * @param {Object} params 设定参数。包括如下可选项
		 * 
		 * - quality : 存储图像的质量，范围是[0,100]。默认值为50。（数字类型）
		 * - targetWidth : 以像素为单位的图像缩放宽度，必须和targetHeight同时使用。相应的宽高比保持不变。默认为无缩放。（数字类型）
		 * - targetHeight : 以像素为单位的图像缩放高度，必须和targetWidth同时使用。相应的宽高比保持不变。默认为无缩放。（数字类型）
		 * 
		 * @method takePicture
		 */
		exports.takePicture = function( successFn, errorFn, params ){
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			!params && ( params = {} );

			//部分参数默认值
			var options = {
				quality: 50,
				//返回图像文件的URI
				destinationType: 1,
				//图像文件的编码方式为JPEG
				encodingType: 0,
				//自动旋转照片，以匹配设备的方向
				correctOrientation: true,
				//设定图片来源为CAMERA
				sourceType: 1,
				//设定所选的媒体类型为PICTURE，只有当sourceType为0或者2时有效
				mediaType: 0,
				//拍照后自动保存到PHOTOALBUM
				saveToPhotoAlbum: true
			};
			
			params = $.extend( {}, options, params || {} );
			
			var errorHandler = function( message ){
				//若为用户取消操作所引起的error，则不返回
				message.indexOf( "cancelled" ) === -1 && errorFn( message );
			}

			navigator.camera.getPicture( successFn, errorHandler || $.noop, params );
		}
		
		/**
		 * 打开设备图片库获取图片
		 * 
		 * @param {Function} successFn 成功回调函数，函数会传入如下参数
		 * 
		 * - imageData : 图像文件的URI。（字符串类型）
		 * 
		 * @param {Function} errorFn 错误回调函数，函数会传入如下参数
		 * 
		 * - message : 设备本地代码提供的错误信息。（字符串类型）
		 * 
		 * @param {Object} params 设定参数。包括如下可选项
		 * 
		 * - quality : 存储图像的质量，范围是[0,100]。默认值为50。（数字类型）
		 * - targetWidth : 以像素为单位的图像缩放宽度，必须和targetHeight同时使用。相应的宽高比保持不变。默认为无缩放。（数字类型）
		 * - targetHeight : 以像素为单位的图像缩放高度，必须和targetWidth同时使用。相应的宽高比保持不变。默认为无缩放。（数字类型）
		 * 
		 * **注意：**只能选取设备中的JPEG和PNG格式图片，否则会执行错误回调函数。
		 * 
		 * @method getPicture
		 */
		exports.getPicture = function( successFn, errorFn, params ){
			successFn = $.isFunction( successFn ) ? successFn : $.noop;
			errorFn = $.isFunction( errorFn ) ? errorFn : $.noop;
			!params && ( params = {} );

			//部分参数默认值
			//设定图片来源为PHOTOLIBRARY。在Android/iOS上设置为0(PHOTOLIBRARY)和2(SAVEDPHOTOALBUM)效果一样。
			params.sourceType = 0;
			params.saveToPhotoAlbum = false;
			//返回图像文件的URI。在Android/iOS上设置为1(URI)和2(NATIVE_URI)效果一样。
			//iOS上返回URL，而Android上返回NativeURL，如"content://media/external/images/media/2528"。
			params.destinationType = 1;

			cordova.require("maf/camera").takePicture( successFn, errorFn, params )
		}
	});


	/**
	 * 音频播放API，包括如下功能
	 * 
	 * - 播放
	 * - 暂停
	 * - 继续
	 * - 停止
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			var playHandler;
	 * 			//播放
	 * 			playHandler = $.maf['native'].audio.play( "/sdcard/Music/白月光.mp3" );
	 * 
	 * 			//暂停
	 * 			$.maf['native'].audio.pause(playHandler);
	 * 
	 * 			//继续
	 * 			$.maf['native'].audio.resume(playHandler);
	 * 
	 * 			//停止
	 * 			$.maf['native'].audio.stop(playHandler);
	 * 		</script>
	 * 
	 * @class $.maf.native.audio
	 */
	cordova.define("maf/audio", function(require, exports, module){

		/**
		 * 播放
		 * 
		 * @param {String} target 音频文件地址
		 * 
		 * @return {Object} 返回音频播放过程的句柄
		 * 
		 * @method play
		 */
		exports.play = function( target ){
			
			if( typeof( target ) !== "string" ) return;
			
			var handler,
				playCompleteFn = function(){ handler.release(); };
		
			handler = new Media( target, playCompleteFn, null );
			
			handler.play();
			
			return handler;
		}
		

		/**
		 * 暂停
		 * 
		 * @param {Object} audioHandler 音频播放过程的句柄
		 * 
		 * @method pause
		 * 
		 */
		exports.pause = function( audioHandler ) {
			audioHandler && audioHandler.pause();
		}
		
		/**
		 * 继续
		 * 
		 * @param {Object} audioHandler 音频播放过程的句柄
		 * 
		 * @method resume
		 * 
		 */
		exports.resume = function( audioHandler ) {
			audioHandler && audioHandler.play();
		}

		/**
		 * 停止
		 * 
		 * @param {Object} audioHandler 音频播放过程的句柄
		 * 
		 * @method stop
		 * 
		 */
		exports.stop = function( audioHandler ) {
			if ( audioHandler ) {
				audioHandler.stop()
				audioHandler.release();
				audioHandler = null;
			}
		}

	});


	/**
	 * push通知服务API
	 * @class $.maf.native.push
	 * 
	 */
	
	cordova.define("maf/push", function(require, exports, module){
		
		/**
		 * 停止push服务
		 * **注意**：停止服务后将不能再收到任何推送通知或者消息
		 * @method stopPush
		 */
		exports.stopPush = function(){
			cordova.exec( null, null, "MAF_PUSH", "stopPush", []);
		}
		
		/**
		 * 设置指定的标签才能收到通知或者消息
		 * @method registerTags
		 * @param {String} tags 标签名称，多个标签名称可以以逗号分隔开
		 */
		exports.registerTags = function( tags ){
			cordova.exec( null, null, "MAF_PUSH", "registerTags", [tags]);
		}
		
		/**
		 * 设置指定的别名才能收到通知或者消息
		 * @method registerAlias
		 * @param {String} alias 别名名称，这个标识应该是独一无二的，比如设备的IMEI序号
		 */
		exports.registerAlias = function( alias ){
			cordova.exec( null, null, "MAF_PUSH", "registerAlias", [alias]);
		}
		
		/**
		 * 重新启动push服务
		 * @method resumePush
		 */
		exports.resumePush = function(){
			cordova.exec( null, null, "MAF_PUSH", "resumePush", []);
		}
		
		/**
		 * 推送消息回调函数
		 * @param {Function} callback
		 * callback会自动传入如下参数：
		 * 
		 * - title 消息的名称
		 * - msg 消息的内容
		 * - extras 附加的信息内容
		 * 
		 * @method registerMsgCallback
		 */
		exports.registerMsgCallback = function( successFn ){
			cordova.exec( successFn || null, null, "MAF_PUSH", "registerMsgCallback", []);
		}
		
		
	});
	
	/**
	 * 浏览pdf文件
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			var readPdf = $.maf["native"].pdf;
	 * 
	 * 			//打开pdf页面
	 * 			slidemenu.showPDF();
	 * 		</script>
	 * 
	 * @class  $.maf.native.pdf
	 */
	cordova.define("maf/pdf", function(require, exports, module){
		
		/**
		 * 打开pdf文件
		 * @param {String} path pdf文件完整路径
		 * @method showPDF
		 */
		exports.showPDF = function(path){
			cordova.exec( null, null, "MAF_PDFReader", "showPDF", [path]);
		}
		
	});
	
	/**
	 * 开启或者关闭MAF的侧滑页面
	 * 
	 * 样例代码如下：
	 * 		<script>
	 * 			function deviceReady(){
	 * 				var deviceApi = $.maf["native"],
	 * 					slidemenu = deviceApi.slidemenu;
	 * 
	 * 				//打开侧滑页面
	 * 				slidemenu.open();
	 * 				//调用主页面的showPage(id)函数
	 * 				slidemenu.invokeMethod("showPage('#secondPage')");
	 * 			}
	 * 
	 * 			$.maf.ready(deviceReady);
	 * 		</script>
	 * 
	 * @class  $.maf.native.slidemenu
	 */
	cordova.define("maf/slidemenu", function(require, exports, module){
		
		/**
		 * 打开侧滑页面
		 * @method open
		 */
		exports.open = function(){
			cordova.exec( function(){
				
			}, null, "MAF_SlideMenu", "open", []);
		}
		
		/**
		 * 关闭侧滑页面
		 * @method close
		 */
		exports.close = function(){
			cordova.exec( null, null, "MAF_SlideMenu", "close", []);
		}
		
		/**
		 * 检测侧滑页面是否打开
		 * @method isOpen
		 * @return {Boolean} 
		 */
		exports.isOpen = function(){
			
			var channel = require('cordova/channel');
			var isOpen = false;
			if(~device.platform.toLowerCase().indexOf("android")){
				cordova.exec( function(res){
					isOpen = res;
				}, null, "MAF_SlideMenu", "isOpen", []);
			}else{
				return window._slidemenu_isopen;
			}
			return isOpen;
		}
		
		/**
		 * 如果侧滑页面处于打开状态则关闭侧滑页面，反之则打开页面
		 * @method toggle
		 */
		exports.toggle = function(){
			cordova.exec( null, null, "MAF_SlideMenu", "toggle", []);
		}
		
		/**
		 * 执行页面上的javascript代码，可以选择执行侧滑页面还是主页面的javascript代码
		 * @method invokeMethod
		 * @param {String} expression javascript表达式，例如"hello()"
		 * @param {Boolean} [bMainPage = true] 可选。设置为true，则调用主页面中的javascript代码；反之，则调用侧滑页面中的javascript代码
		 */
		exports.invokeMethod = function(expression, bMainPage){
			bMainPage !== false && ( bMainPage = true );
			cordova.exec( null, null, "MAF_SlideMenu", "invokeMethod", [expression, Number(bMainPage)]) ;
		}
		
	});
	
	//获取设备常量信息
	// 
	// 样例代码如下：
	// 		<script>
	// 			function deviceReady(){
	// 				alert($.maf["native"].constant.updateUrl)
	// 			}
	// 
	// 			$.maf.ready(deviceReady);
	// 		</script>
	// 
	// @class  $.maf.native.constants
	//
    cordova.define("maf/Constants", function(require, exports, module) {

        var argscheck = require('cordova/argscheck'),
            channel = require('cordova/channel'),
            utils = require('cordova/utils'),
            exec = require('cordova/exec');

        // Tell cordova channel to wait on the CordovaInfoReady event
        channel.waitForInitialization('onCordovaInfoReady');

        //This represents the mobile device, and provides properties for inspecting the model, version, UUID of the
        //phone, etc.
        //@constructor
        //
        function Constants() {
            this.available = false;
            this.versionInfo = null;
            this.updateUrl = null;
            this.filePath = null;

            var me = this;
            
            channel.onCordovaReady.subscribe(function() {
                me.getVersion(function(version) {
                    me.versionInfo = version;
                    channel.onCordovaInfoReady.fire();
                },function(e) {
                    utils.alert("[ERROR] Error initializing Cordova: " + e);
                });
                
                me.getUpdateUrl(function(url) {
                    me.updateUrl = url;
                    channel.onCordovaInfoReady.fire();
                },function(e) {
                    utils.alert("[ERROR] Error initializing Cordova: " + e);
                });
            });
        }

        Constants.prototype.getVersion = function(successCallback, errorCallback) {
            exec(successCallback, errorCallback, "MAF_Device", "getVersion", []);
        };
        
        Constants.prototype.getUpdateUrl = function(successCallback, errorCallback) {
            exec(successCallback, errorCallback, "MAF_Downloader","getUpdateUrl",[]);
        };

        module.exports = new Constants();

    });
	
})( jQuery );



